Unit MainFFD;
{****************************************************************************}
{* MainFFD unit                                                             *}
{*                                                                          *}
{* Main unit of the example. See the project .dpr file for a description    *}
{* of the conduit.                                                          *}
{****************************************************************************}

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, ffdb, pfgPalmDb, pfgBaseClasses, pfgPalmComp, pfgPalmSyncComponent,
  ffllbase, ffdbbase;

Type
  TDataModule1 = Class(TDataModule)
    PalmComponent1: TpfgPalmComponent;
    PalmRemoteTable1: TpfgPalmRemoteTable;
    PalmSyncComponent1: TpfgPalmSyncComponent;
    ffCommsEngine1: TffCommsEngine;
    ffSession1: TffSession;
    ffDatabase1: TffDatabase;
    TblTest_: TffTable;
    TblTest_ItemType: TStringField;
    TblTest_ParentID: TIntegerField;
    TblTest_Data: TStringField;
    TblTest_id: TIntegerField;
    TblTest_dirty: TBooleanField;
    TblTest_deleted: TBooleanField;
    TblTest_attr: TIntegerField;
    Procedure DataModuleCreate(Sender: TObject);
    Procedure DataModuleDestroy(Sender: TObject);
    Procedure PalmComponent1ExecConduit(Const Props: TpfgSyncProperties);
    Procedure PalmSyncComponent1BeforeLocalList(Sender: TObject; Const Props: TpfgSyncProperties);
    Function  PalmSyncComponent1BeforeSync(Sender: TObject; Const Props: TpfgSyncProperties): Integer;
    Function  PalmSyncComponent1CompareRecord(Sender: TObject; Const Props: TpfgSyncProperties; Const PalmFlags, PCFlags: Byte): TRecordAction;
    Procedure PalmSyncComponent1EmptyLocalTable(DataSet: TDataSet);
    Procedure PalmSyncComponent1ErrorMessage(Sender: TObject; Const ErrorMessage: String);
    Function  PalmSyncComponent1FindLocalRecordID(Sender: TObject; RecordID: Cardinal): Boolean;
  Private
    { Private declarations }
  Public
    { Public declarations }
  End;

var
  DataModule1: TDataModule1;

Implementation

{$R *.DFM}

Uses
  pfgSyncMgr;

Var
  LogList: TStringList= Nil; // Our own log list

Const
  txtLogFile='W:\D6\_Test\PPConduit\SchPlg.log';  // The path to our log file
  txtFFServerName='FFDevelop@192.168.1.5';        // The FlashFiler ServerName and IP address
  txtFFAliasName='JobLog';                        // The FlashFiler Alias/Databasename
  txtFFTableName='test';                          // The FlashFiler tablename
  txtPPConduitName='FlashFiler Sample conduit';   // The text that appear when you HotSync.
  txtPPCreatorID='FFSS';                          // The creator ID, remember that this is case sensitive and must be 4 characters long!
  txtPPDirectory='FFSyncSample';                  // The directory for our data in the users folder, this is not used in this sample!
  txtPPTableName='test';                          // The name of the table on the PalmPilot

Procedure LogIt(Value: String; SetTimeStamp: Boolean= True);
Begin
  // This procedure adds the specified value to our log stringlist
  // If the SetTimeStamp is true then the current date and time will be added
  // to the beginning of the string added, the timestamp format is system dependant.
  If Assigned(LogList) And (Value<>'') Then
  Begin
    If SetTimeStamp Then
      LogList.Add(DateTimeToStr(Now)+' '+Value)
    Else
      LogList.Add(Value);
  End;
End;

Procedure TDataModule1.DataModuleCreate(Sender: TObject);
Begin
  // This happends when the DLL loads, it is genrally NOT a good idea to do ANY
  // dtabase stuff here, but later on inside the conduits loop.
  // So as you can see there is nothing mutch going on here, but if you have some
  // variables or so that needs to be initialized, then this is the place.
  // We will primary use this to write to the loag and set the information we need
  // on the FlashFiler stuff and some other constants.
  LogIt('DataModule.OnCreate.');
  ffCommsEngine1.ServerName:=txtFFServerName;   // The IP address and and name of our FlashFiler server
  ffDatabase1   .AliasName :=txtFFAliasName;    // The Alias / Dtabase name we wan't to use
  TblTest_      .TableName :=txtFFTableName;    // And the tablename that we want to synchronize with
  PalmComponent1.ConduitName:=txtPPConduitName; // The text that appear when you HotSync.
  PalmComponent1.CreatorID  :=txtPPCreatorID;   // The creator ID, remember that this is case sensitive and must be 4 characters long!
  PalmComponent1.Directory  :=txtPPDirectory;   // The directory for our data in the users folder, this is not used in this sample!
  PalmRemoteTable1.CreatorID:=txtPPCreatorID;   // The creator ID, remember that this is case sensitive and must be 4 characters long!
  PalmRemoteTable1.TableName:=txtPPTableName;   // The name of the table on the PalmPilot
End;

Procedure TDataModule1.DataModuleDestroy(Sender: TObject);
Begin
  // This happends just before the DLL is unloaded!
  // If you "Runtime" created anything, the this is a good place to make sure that
  // your memory is released!
  // We haven't created anything so we will only add a notification to the log
  LogIt('DataModule.OnDestroy.');
End;

Procedure TDataModule1.PalmComponent1ExecConduit(Const Props: TpfgSyncProperties);
Var
  AProps: TpfgSyncProperties;
Begin
  // This happends when HotSync want's to talk with your creator id!
  // Whatever you wan't to do with the Palm and it's data, then it happends in here
  // In this sample we start by initializing our FlashFiler stuff and finally
  // start a synchronization!
  // Note how carefully I have divided the FlashFiler initializtion, the order of the
  // initilization is very important, you can of course do it all in a try finally,
  // I just did it this way to make sure that everything went Ok!
  LogIt('PalmComponent1ExecConduit');
  Try
    ffCommsEngine1.Open;
    LogIt('ffCommsEngine1.Open');
    If ffCommsEngine1.Active Then
    Try
      ffSession1.Open;
      LogIt('ffSession1.Open');
      If ffSession1.Active Then
      Try
        ffDatabase1.Open;
        LogIt('ffDatabase1.Open');
        If ffDatabase1.Connected Then
        Try
          LogIt('TblTest_.Open');
          // To make the PalmSyncComponent1 synchronize (not copy) the tables, we need to change
          // the Props.m_SyncType property
          AProps:=Props;
          AProps.m_SyncType:=eSlow; // Setting it to slow forces the PalmSyncComponent1 to check every record!
          PalmSyncComponent1.Sync(AProps);
          TblTest_.Close;
          LogIt('TblTest_.Close');
        Except
          LogIt('Exception TblTest_');
        End;
        ffDatabase1.Close;
        LogIt('ffDatabase1.Close');
      Except
        LogIt('Exception ffDatabase1');
      End;
      ffSession1.Close;
      LogIt('ffSession1.Close');
    Except
      LogIt('Exception ffSession1');
    End;
    ffCommsEngine1.Close;
    LogIt('ffCommsEngine1.Close');
  Except
    LogIt('Exception ffCommsEngine1');
  End;
End;

Procedure TDataModule1.PalmSyncComponent1BeforeLocalList(Sender: TObject; Const Props: TpfgSyncProperties);
Begin
  // This happends when the HotSync asks us for a local list (local table) so we can open it or just put it
  // in it's right position or even rnage/filter our data!
  LogIt('PalmSyncComponent1BeforeLocalList');
  If Not TblTest_.Active Then
    TblTest_.Open;
  If TblTest_.Active Then
  Try
    TblTest_.Refresh;
    TblTest_.First;
  Except
    LogIt('PalmSyncComponent1BeforeLocalList - Unable to reposition table!');
  End;
End;

Function  TDataModule1.PalmSyncComponent1BeforeSync(Sender: TObject; Const Props: TpfgSyncProperties): Integer;
Begin
  // This happends just before HotSync is going to do a compare on each record, do not filter or range your data
  // here, since you might get unwanted results because HotSync will copy the records to or from the Palm if they
  // arent located, instead make a event handler for the OnCompareRecord.
  // As OnBeforeLocalList, this is where you open your local table.
  Result:=-1; // The result is not used in the code of TpfgPalmSyncComponent :-\, so we just set it to -1 and make Delphi SHUT UP!
  LogIt('PalmSyncComponent1BeforeSync');
  // We could make a ff table creation here and so on, if you do, then remember to remove
  // the TblTest_.Open from PalmComponent1ExecConduit !!! but not TblTest_.Close, unless
  // you know that it is closed when the PalmSyncComponent1 is finished.
  TblTest_.Open;
End;

Function  TDataModule1.PalmSyncComponent1CompareRecord(Sender: TObject; Const Props: TpfgSyncProperties; Const PalmFlags, PCFlags: Byte): TRecordAction;
Begin
  // This event - as you probably allready have guessed - is fired each time HotSync compares each record (PC vs. Palm)
  // HotSync normally handles this event, I just created it to show what you could do with it.
  // The best would be if we had a timestamp on the record, so we could see wether the PC or the Palm has the newest record,
  // but that is not the case here!
  LogIt('PalmSyncComponent1CompareRecord SyncType: '+IntToStr(Props.m_SyncType)+' PalmFlags: '+IntToStr(PalmFlags)+' PCFlags: '+IntToStr(PCFlags));
  // We start by making sure HotSync does nothing with both records, in case anything went wrong.
  Result:=DoNothing;
  // By comparing the bit's in the flags, we can see what have happened on the records
  If ((PalmFlags And eRecAttrDeleted) = eRecAttrDeleted) Or ((PCFlags And eRecAttrDeleted) = eRecAttrDeleted) Then
    Result:=DeletePalmPC
  Else
  If ((PalmFlags And eRecAttrDirty) = eRecAttrDirty) And (PCFlags=0) Then
    Result:=CopyToPC
  Else
  If (PalmFlags=0) And ((PCFlags And eRecAttrDirty) = eRecAttrDirty) Then
    Result:=CopyToPalm
  Else
  If ((PalmFlags And eRecAttrDirty) = eRecAttrDirty) And ((PCFlags And eRecAttrDirty) = eRecAttrDirty) Then
    Result:=CopyToPalm;
  LogIt('PalmSyncComponent1CompareRecord Resulted in '+IntToStr(Integer(Result)));
End;

Procedure TDataModule1.PalmSyncComponent1EmptyLocalTable(DataSet: TDataSet);
Begin
  // This actually only happends if the table on the Palm has a request for deletion or if you specific call
  // PalmSyncComponent1.DoEmptyLocalTable;, all it does is requesting you to empty your table!
  LogIt('PalmSyncComponent1EmptyLocalTable');
  // Make sure the table is open before deleting anything!
  If Not DataSet.Active Then
    DataSet.Open;
  DataSet.First;
  While Not DataSet.EOF Do
    DataSet.Delete;
End;

Procedure TDataModule1.PalmSyncComponent1ErrorMessage(Sender: TObject; Const ErrorMessage: String);
Begin
  // This event occurs if any error should occur, in this case it is merely a information for us to
  // give on to the user.
  LogIt('PalmSyncComponent1ErrorMessage: '+ErrorMessage);
  PalmComponent1.Information:=ErrorMessage;
End;

Function  TDataModule1.PalmSyncComponent1FindLocalRecordID(Sender: TObject; RecordID: Cardinal): Boolean;
Begin
  // This event occurs when HotSync tries to find a corresponding record in your data, normally
  // PalmSyncComponent component handles this by checking the LocalRecIdField property.
  // All I do is locate the record and return True if I found it otherwise False.
  LogIt('PalmSyncComponent1FindLocalRecordID');
  If TblTest_.Active Then
    Result:=TblTest_.Locate('id', RecordID, [])
  Else
    Result:=False;
End;

Initialization
  // This is where it all begins!, it enables the DLL to function!
  // We first establish our log (NOTE! this log has NOTHING to do with the conduits or HotSync!
  // it's primary function is to avoid debugging all the time! Remark the .Create if you do not need
  // the log, remember to set the path where you wan't to store the log file!!!!!
  // The reason we have the log outside the DataModule is that we can add to the log without
  // consern of the rest of this unit, classes etc. .
  Try
    // Create the stringlist
    LogList:=TStringList.Create;
    // Load previously stored logs so we don't overwrite it
    If FileExists(txtLogFile) Then
      LogList.LoadFromFile(txtLogFile);
  Except
    LogList:=Nil;
  End;
  LogIt('Loglist created.');
  // Create the datamodule, the components itself and the exported functions will
  // automticly be called from HotSync
  DataModule1 := TDataModule1.Create(Nil);
Finalization
  // And - you guessed it - This is where it all ends
  // Release the datamodule, this is important since we created it without a owner (NIL)
  FreeAndNil(DataModule1);
  LogIt('Loglist destroying.');
  // Save our log, this will overwrite any previously stored log, that's why we loaded it above!
  If Assigned(LogList) Then
    LogList.SaveToFile(txtLogFile);
  // And finally we release our log
  FreeAndNil(LogList);
End.
